@@ -132,7 +132,7 @@ typedef BOOL (*PIsWow64Process2)(HANDLE, USHORT*, USHORT*);
 
 
 USHORT
-_getNativeMachine()
+_getNativeMachine(void)
 {
     static USHORT _nativeMachine = IMAGE_FILE_MACHINE_UNKNOWN;
     if (_nativeMachine == IMAGE_FILE_MACHINE_UNKNOWN) {
@@ -163,14 +163,14 @@ _getNativeMachine()
 
 
 bool
-isAMD64Host()
+isAMD64Host(void)
 {
     return _getNativeMachine() == IMAGE_FILE_MACHINE_AMD64;
 }
 
 
 bool
-isARM64Host()
+isARM64Host(void)
 {
     return _getNativeMachine() == IMAGE_FILE_MACHINE_ARM64;
 }
@@ -492,10 +492,14 @@ dumpSearchInfo(SearchInfo *search)
         return;
     }
 
-#define DEBUGNAME(s) L"SearchInfo." ## s
-#define DEBUG(s) debug(DEBUGNAME(#s) L": %s\n", (search->s) ? (search->s) : L"(null)")
-#define DEBUG_2(s, sl) _debugStringAndLength((search->s), (search->sl), DEBUGNAME(#s))
-#define DEBUG_BOOL(s) debug(DEBUGNAME(#s) L": %s\n", (search->s) ? L"True" : L"False")
+#ifdef __clang__
+#define DEBUGNAME(s) L # s
+#else
+#define DEBUGNAME(s) # s
+#endif
+#define DEBUG(s) debug(L"SearchInfo." DEBUGNAME(s) L": %s\n", (search->s) ? (search->s) : L"(null)")
+#define DEBUG_2(s, sl) _debugStringAndLength((search->s), (search->sl), L"SearchInfo." DEBUGNAME(s))
+#define DEBUG_BOOL(s) debug(L"SearchInfo." DEBUGNAME(s) L": %s\n", (search->s) ? L"True" : L"False")
     DEBUG(originalCmdLine);
     DEBUG(restOfCmdLine);
     DEBUG(executablePath);
@@ -561,21 +565,6 @@ findArgv0End(const wchar_t *buffer, int bufferLength)
  ***                          COMMAND-LINE PARSING                          ***
 \******************************************************************************/
 
-// Adapted from https://stackoverflow.com/a/65583702
-typedef struct AppExecLinkFile { // For tag IO_REPARSE_TAG_APPEXECLINK
-    DWORD reparseTag;
-    WORD reparseDataLength;
-    WORD reserved;
-    ULONG version;
-    wchar_t stringList[MAX_PATH * 4];  // Multistring (Consecutive UTF-16 strings each ending with a NUL)
-    /* There are normally 4 strings here. Ex:
-        Package ID:  L"Microsoft.DesktopAppInstaller_8wekyb3d8bbwe"
-        Entry Point: L"Microsoft.DesktopAppInstaller_8wekyb3d8bbwe!PythonRedirector"
-        Executable:  L"C:\Program Files\WindowsApps\Microsoft.DesktopAppInstaller_1.17.106910_x64__8wekyb3d8bbwe\AppInstallerPythonRedirector.exe"
-        Applic. Type: L"0"   // Integer as ASCII. "0" = Desktop bridge application; Else sandboxed UWP application
-    */
-} AppExecLinkFile;
-
 
 int
 parseCommandLine(SearchInfo *search)
@@ -767,55 +756,6 @@ _shebangStartsWith(const wchar_t *buffer, int bufferLength, const wchar_t *prefi
 }
 
 
-int
-ensure_no_redirector_stub(wchar_t* filename, wchar_t* buffer)
-{
-    // Make sure we didn't find a reparse point that will open the Microsoft Store
-    // If we did, pretend there was no shebang and let normal handling take over
-    WIN32_FIND_DATAW findData;
-    HANDLE hFind = FindFirstFileW(buffer, &findData);
-    if (!hFind) {
-        // Let normal handling take over
-        debug(L"# Did not find %s on PATH\n", filename);
-        return RC_NO_SHEBANG;
-    }
-
-    FindClose(hFind);
-
-    if (!(findData.dwFileAttributes & FILE_ATTRIBUTE_REPARSE_POINT &&
-        findData.dwReserved0 & IO_REPARSE_TAG_APPEXECLINK)) {
-        return 0;
-    }
-
-    HANDLE hReparsePoint = CreateFileW(buffer, 0, FILE_SHARE_READ, NULL, OPEN_EXISTING, FILE_FLAG_OPEN_REPARSE_POINT, NULL);
-    if (!hReparsePoint) {
-        // Let normal handling take over
-        debug(L"# Did not find %s on PATH\n", filename);
-        return RC_NO_SHEBANG;
-    }
-
-    AppExecLinkFile appExecLink;
-
-    if (!DeviceIoControl(hReparsePoint, FSCTL_GET_REPARSE_POINT, NULL, 0, &appExecLink, sizeof(appExecLink), NULL, NULL)) {
-        // Let normal handling take over
-        debug(L"# Did not find %s on PATH\n", filename);
-        CloseHandle(hReparsePoint);
-        return RC_NO_SHEBANG;
-    }
-
-    CloseHandle(hReparsePoint);
-
-    const wchar_t* redirectorPackageId = L"Microsoft.DesktopAppInstaller_8wekyb3d8bbwe";
-
-    if (0 == wcscmp(appExecLink.stringList, redirectorPackageId)) {
-        debug(L"# ignoring redirector that would launch store\n");
-        return RC_NO_SHEBANG;
-    }
-
-    return 0;
-}
-
-
 int
 searchPath(SearchInfo *search, const wchar_t *shebang, int shebangLength)
 {
@@ -877,11 +817,6 @@ searchPath(SearchInfo *search, const wchar_t *shebang, int shebangLength)
         return RC_BAD_VIRTUAL_PATH;
     }
 
-    int result = ensure_no_redirector_stub(filename, buffer);
-    if (result) {
-        return result;
-    }
-
     // Check that we aren't going to call ourselves again
     // If we are, pretend there was no shebang and let normal handling take over
     if (GetModuleFileNameW(NULL, filename, MAXLEN) &&
@@ -1923,7 +1858,6 @@ struct AppxSearchInfo {
 
 struct AppxSearchInfo APPX_SEARCH[] = {
     // Releases made through the Store
-    { L"PythonSoftwareFoundation.Python.3.13_qbz5n2kfra8p0", L"3.13", 10 },
     { L"PythonSoftwareFoundation.Python.3.12_qbz5n2kfra8p0", L"3.12", 10 },
     { L"PythonSoftwareFoundation.Python.3.11_qbz5n2kfra8p0", L"3.11", 10 },
     { L"PythonSoftwareFoundation.Python.3.10_qbz5n2kfra8p0", L"3.10", 10 },
@@ -1933,7 +1867,6 @@ struct AppxSearchInfo APPX_SEARCH[] = {
     // Side-loadable releases. Note that the publisher ID changes whenever we
     // renew our code-signing certificate, so the newer ID has a higher
     // priority (lower sortKey)
-    { L"PythonSoftwareFoundation.Python.3.13_3847v3x7pw1km", L"3.13", 11 },
     { L"PythonSoftwareFoundation.Python.3.12_3847v3x7pw1km", L"3.12", 11 },
     { L"PythonSoftwareFoundation.Python.3.11_3847v3x7pw1km", L"3.11", 11 },
     { L"PythonSoftwareFoundation.Python.3.11_hd69rhyc2wevp", L"3.11", 12 },
@@ -2014,8 +1947,7 @@ struct StoreSearchInfo {
 
 
 struct StoreSearchInfo STORE_SEARCH[] = {
-    { L"3", /* 3.12 */ L"9NCVDN91XZQP" },
-    { L"3.13", L"9PNRBTZXMB4Z" },
+    { L"3", /* 3.11 */ L"9NRWMJP3717K" },
     { L"3.12", L"9NCVDN91XZQP" },
     { L"3.11", L"9NRWMJP3717K" },
     { L"3.10", L"9PJPW5LDXLZ5" },
@@ -2541,8 +2473,7 @@ launchEnvironment(const SearchInfo *search, const EnvironmentInfo *launch, wchar
     window, or fetching a message).  As this launcher doesn't do this
     directly, that cursor remains even after the child process does these
     things.  We avoid that by doing a simple post+get message.
-    See http://bugs.python.org/issue17290 and
-    https://bitbucket.org/vinay.sajip/pylauncher/issue/20/busy-cursor-for-a-long-time-when-running
+    See http://bugs.python.org/issue17290
     */
     MSG msg;
 
